#include "testlib.h"

#include <vector>
#include <utility>
#include <algorithm>

enum { MAX = 100 };

int N, M, field[MAX][MAX];
std::vector<std::pair<int, int>> snake;

bool
in_field(int x, int y)
{
    return 0 <= x && x < N && 0 <= y && y < M;
}

int
meals_count()
{
    int c = 0;
    for (int i = 0; i < N; ++i) {
        for (int j = 0; j < M; ++j) {
            if (field[i][j] == 1) {
                ++c;
            }
        }
    }
    return c;
}

enum Action { UP = 0, DOWN = 2, LEFT = 1, RIGHT = 3 };

void
read_field()
{
    N = inf.readInt();
    M = inf.readInt();

    snake.clear();
    size_t count_of_2 = 0;
    for (int i = 0; i < N; ++i) {
        for (int j = 0; j < M; ++j) {
            field[i][j] = inf.readInt();
            if (field[i][j] == 3) {
                snake.emplace_back(i, j);
                field[i][j] = 0;
            } else if (field[i][j] == 2) {
                ++ count_of_2;
                field[i][j] = 22;
            }
        }
    }

    int last_x = snake[0].first;
    int last_y = snake[0].second;

    while (count_of_2-- > 0) {
        int next_x, next_y;

        for (int action = 0; action <= 3; ++action) {
            next_x = (action == DOWN ? last_x + 1 : action == UP ? last_x - 1 : last_x);
            next_y = (action == LEFT ? last_y - 1 : action == RIGHT ? last_y + 1 : last_y);
            if (in_field(next_x, next_y) && field[next_x][next_y] == 22) {
                snake.emplace_back(next_x, next_y);
                break;
            }
        }
        field[next_x][next_y] = 0;
    }
}

void
move_snake(Action action)
{
    int last_x = snake[0].first;
    int last_y = snake[0].second;
    int next_x = (action == DOWN ? last_x + 1 : action == UP ? last_x - 1 : last_x);
    int next_y = (action == LEFT ? last_y - 1 : action == RIGHT ? last_y + 1 : last_y);
    auto next = std::pair<int, int>(next_x, next_y);
    
    if (! in_field(next_x, next_y)) {
        quitf(_wa, "snake goes outside of the field to (%d, %d)", next_x, next_y);
    }
    
    if (std::count(snake.begin(), snake.end(), next) > 0) {
        quitf(_wa, "snake tryes to eat himself to (%d, %d)", next_x, next_y);
    }

    if (field[next_x][next_y] == 1) {
        field[next_x][next_y] = 0;
    } else {
        snake.pop_back();
    }
    snake.emplace(snake.begin(), next_x, next_y);
}

int
main(int argc, char *argv[])
{
    setName("check the snake route");
    registerTestlibCmd(argc, argv);

    read_field();

    if (ans.readInt() == -1) {
        if (ouf.readInt() != -1) {
            quitf(_wa, "expected -1, but gotten another result");
        } else {
            quitf(_ok, "answer is right");
        } 
    }

    while (!ouf.seekEof()) {
        int action = ouf.readInt();
        if (action < 0 || action > 3) {
            quitf(_pe, "unexpected action %d", action);
        }

        move_snake(static_cast<Action>(action));
    }

    int c;
    if ((c = meals_count()) == 0) {
        quitf(_ok, "ok");
    } else{
        quitf(_wa, "not all cells with meal were eaten, %d stay", c);
    }
}
